"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path
# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open

import glob

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'PyPI.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    include_package_data=True,

    # This is the name of your project. The first time you publish this
    # package, this name will be registered for you. It will determine how
    # users can install this project, e.g.:
    #
    # $ pip install sampleproject
    #
    # And where it will live on PyPI: https://pypi.org/project/sampleproject/
    #
    # There are some restrictions on what makes a valid project name
    # specification here:
    # https://packaging.python.org/specifications/core-metadata/#name
    name='microspec',  # Required

    # Versions should comply with PEP 440:
    # https://www.python.org/dev/peps/pep-0440/
    #
    # For a discussion on single-sourcing the version across setup.py and the
    # project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.1.1a6',  # Required (reset to this before PyPI publish)
    # version='0.1.1a13',  # Temporary increment for TestPyPI publish)

    # This is a one-line description or tagline of what your project does. This
    # corresponds to the "Summary" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#summary
    description='Python API for the Chromation spectrometer dev-kit',

    # This is an optional longer description of your project that represents
    # the body of text which users will see when they visit PyPI.
    #
    # Often, this is the same as your README, so you can just read it in from
    # that file directly (as we have already done above)
    #
    # This field corresponds to the "Description" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-optional
    long_description=long_description,

    # Denotes that our long_description is in Markdown; valid values are
    # text/plain, text/x-rst, and text/markdown
    #
    # Optional if long_description is written in reStructuredText (rst) but
    # required for plain-text or Markdown; if unspecified, "applications should
    # attempt to render [the long_description] as text/x-rst; charset=UTF-8 and
    # fall back to text/plain if it is not valid rst" (see link below)
    #
    # This field corresponds to the "Description-Content-Type" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#description-content-type-optional
    long_description_content_type='text/markdown',

    # This should be a valid link to your project's main homepage.
    #
    # This field corresponds to the "Home-Page" metadata field:
    # https://packaging.python.org/specifications/core-metadata/#home-page-optional
    # url='https://www.chromation.com/',
    url='https://github.com/microspectrometer/microspec',


    # This should be your name or the name of the organization which owns the
    # project.
    author='Chromation',
    author_email="mike@chromationspec.com",

    # This should be a valid email address corresponding to the author listed
    # above.
    #author_email='pypa-dev@googlegroups.com',  # Optional

    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        #'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        # These classifiers are *not* checked by 'pip install'. See instead
        # 'python_requires' below.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',

        # Additional classifiers
        "Topic :: Scientific/Engineering",
        "Topic :: Software Development :: Embedded Systems",
        "Topic :: Terminals :: Serial",
    ],

    # This field adds keywords for your project which will appear on the
    # project page. What does your project relate to?
    #
    # Note that this is a string of words separated by whitespace, not a list.
    keywords='spectrometer API USB serial hardware embedded-systems',

    # When your source code is in a subdirectory under the project root, e.g.
    # `src/`, it is necessary to specify the `package_dir` argument.
    package_dir={'': 'src'},

    # You can just specify package directories manually here if your project is
    # simple. Or you can use find_packages().
    #
    # Alternatively, if you just want to distribute a single Python file, use
    # the `py_modules` argument instead as follows, which will expect a file
    # called `my_module.py` to exist:
    #
    #   py_modules=["my_module"],
    #
    packages=find_packages(where='src'),  # Required

    # Specify which Python versions you support. In contrast to the
    # 'Programming Language' classifiers above, 'pip install' will check this
    # and refuse to install the project if the version does not match. If you
    # do not support Python 2, you can simplify this to '>=3.5' or similar, see
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#python-requires
    python_requires='>=3.7', # need 3.7 for namedtuple kwarg "defaults"

    # This field lists other packages that your project depends on to run.
    # Any package you put here will be installed by pip when your project is
    # installed, so they must be valid existing projects.
    #
    # For an analysis of "install_requires" vs pip's requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['pyserial'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). Users will be able to install these using the "extras"
    # syntax, for example:
    #
    #   $ pip install sampleproject[dev]
    #
    # Similar to `install_requires` above, these must be valid existing
    # projects.
    extras_require={
        # For developers:
        # emulate hardware (Linux/Mac only, requires socat)
        # run unit tests after modifying API
        # rebuild documentation after modifying API
        'dev': [
            'sphinx', # docs
            'sphinx-rtd-theme', # docs
            'recommonmark', # docs
            'm2r', # docs
            'sphinxcontrib-argdoc', # docs
            'pytest', # docs and unit test
            'pytest-testdox', # pretty-print unit tests
            'tabulate', # docs and unit test
            'psutil', # docs and emulator (requires socat)
            ],
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.
    #
    # If using Python 2.6 or earlier, then these have to be included in
    # MANIFEST.in as well.
    #package_data={
    #    'sample': ['package_data.dat'],
    #},
    # DO NOT USE package_data AND MANIFEST.in AT THE SAME TIME

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files
    #
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    # For example, if installing in virtual environment `foo`,
    # The microspec package is installed in
    # foo\Lib\site-packages\microspeclib
    # But these `data_files` are installed outside the package:
    # - the tests are in foo/share/microspeclib/tests
    # - the docs are in foo/share/microspeclib/docs
    # - the JSON file is in foo/share/microspeclib/cfg
    data_files=[
      ('share/microspeclib/cfg',                     ['cfg/microspec.json']),
      ('share/microspeclib/tests',                   glob.glob("tests/*.*")),
      ('share/microspeclib/doc/build/html',          glob.glob("doc/build/html/*.*")),
      ('share/microspeclib/doc/build/html/_static',  glob.glob("doc/build/html/_static/*.*")),
    ],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # `pip` to create the appropriate form of executable for the target
    # platform.
    #
    # For example, the following would provide a command called `sample` which
    # executes the function `main` from this package when invoked:
    #entry_points={  # Optional
    #    'console_scripts': [
    #        'sample=sample:main',
    #    ],
    #},
    # example: run from PowerShell: `> microspec_simple_example.py`
    # On Windows, creates foo\Scripts\simple_example.py
    scripts=['bin/microspec_cmdline.py',
        'bin/microspec_emulator.py',
        'bin/microspec_expert_example.py',
        'bin/microspec_simple_example.py'
    ],
    # entry_points style requires `bin` is a module and `main` is
    # a function.
    # entry_points={
    #     "console_scripts": [
    #         "simple_example=bin.microspec_simple_example:main",
    #     ],
    # but `bin` is not a module and `main` is not a function.
    # Copy script to new directory `microspeclib/examples`.
    # Rename as 'microspec_api.py'.
    # The script name cannot contain "-", use "_" instead.
    # `import *` is not allowed inside a function call.
    # The entry point key must be "console_scripts". This:
    # - installs the .exe in the {venv}\Scripts folder on Windows
    # - installs the script in the {venv}/bin folder on Linux
    # Run from PowerShell: `> microspec-cmdline`
    # Run from bash: `$ microspec-cmdline`
    # On Windows, install creates {venv}\Scripts\microspec-api-example.exe
    entry_points={
        "console_scripts": [
            "microspec-api-example=microspeclib.examples.microspec_api:main",
            "microspec-lowlevel-api-example=microspeclib.examples.microspec_lowlevel_api:main",
            "microspec-cmdline=microspeclib.scripts.microspec_cmdline:main",
        ],
    },

    # List additional URLs that are relevant to your project as a dict.
    #
    # This field corresponds to the "Project-URL" metadata fields:
    # https://packaging.python.org/specifications/core-metadata/#project-url-multiple-use
    #
    # Examples listed include a pattern for specifying where the package tracks
    # issues, where the source is hosted, where to say thanks to the package
    # maintainers, and where to support the project financially. The key is
    # what's used to render the link text on PyPI.
    project_urls={
        'Chromation': 'https://www.chromation.com/',
        'Thanks To': 'http://eruciform.com/',
    },
    license='MIT', # field in *.egg-info/PKG-INFO
    platforms=['Windows', 'Mac', 'Linux'], # legacy field in *.egg-info/PKG-INFO
)
